/**
 * The main file 
 * It contains the calls to read the XML instance 
 * and process the data to output in text format
 * as described in the Exam document.
 */

import java.io.File;
import java.io.*;
import java.io.IOException;

import java.util.List;
import java.util.HashMap;
import java.util.Random;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


class ColorPrg {
	public Integer heat;
	public Integer clear;
	public ColorPrg() 
	{
	    heat=0;
	    clear=0;
	}
}

public class ProblemLoader {

    /*
     * CONSTANTS
     */

    public static String MAX_SAME_POS_TIME = "max_same_pos_time";

    public static String CRANE_DOWN_TIME = "crane_down_time";

    public static String CRANE_UP_TIME = "crane_up_time";

    public static String DEBUG = "debug";

    public static String ABS = "abs_in_mm"; // 6.39mm, 0.639 cm

    public static String MIN_CRANE_DISTANCE_ABS = "min_crane_distance_abs";

    public static String MIN_CRANE_DISTANCE_MM = "min_crane_distance_mm";

    public static String CRANE_START_UP = "crane_start_up";

    public static String COLOR_PARALLEL_TANK = "color_parallel_station";

    public static String ACCELERATION = "acceleration"; // 600 mm/s^2

    public static String SPEED_STEPS_IN_ABS = "deacceleration_steps"; // =

    // {0,
    // 30,
    // 60,
    // 90,
    // 100,
    // 270,
    // 350,
    // Integer.MAX_VALUE};

    public static final String SPEEDS_IN_MM_S = "deacceleration_speeds"; // =

    // {0,
    // 54.5455,
    // 163.636,
    // 272.727,
    // 381.818,
    // 490.909,
    // 954.545,
    // 1200.00};

    public static Random r;

    public static Integer crane_down_time;

    public static Integer crane_up_time;

    public static Integer parallel_color_tank;

    public static Double ABS_conv;

    public static Double acceleration;

    public static List<Double> speeds_in_ABS;

    public static List<Double> speeds_in_mm;

    private static HashMap<Integer, Integer> ABS_tanks;

    private static HashMap<Integer, Integer> drip_tanks;

    private static HashMap<Integer, List<Integer>> parallel_tanks;

    private static HashMap<Integer, Integer> color_bar;

    
    private static HashMap<Integer, ColorPrg> colors;

    public static void loadFromXML(String filename) {
	File file = new File(filename);
	try {
	    loadFromXML(file);
	} catch (Exception e) {
	    System.out.println(e.getMessage());
	}

    }

    public static void loadFromXML(File file) throws Exception {

	if (!file.exists() || !file.canRead()) {
	    throw new Exception("Could not find or read the problem file: "
		    + file.getAbsolutePath());
	}

	DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	// XPath xpath = XPathFactory.newInstance().newXPath();
	try {
	    DocumentBuilder db = dbf.newDocumentBuilder();
	    Document document = db.parse(file);

	    loadFromXML(document);

	} catch (ParserConfigurationException e) {
	    throw new Exception(e.getMessage());
	} catch (SAXException e) {
	    throw new Exception(e.getMessage());
	} catch (IOException e) {
	    throw new Exception(e.getMessage());
	}

    }

    public static void loadFromXML(Document document) {

	Element root = document.getDocumentElement();

	NodeList nl = null;
	nl = root.getElementsByTagName("constants");
	loadConstants(nl);

	NodeList nl2 = root.getElementsByTagName("tank");
	loadTankManager(nl2);

	nl2 = root.getElementsByTagName("colorprogram");
	loadColorRecipes(nl2);

	nl2 = root.getElementsByTagName("crane");
	loadCraneManager(nl2);

	nl2 = root.getElementsByTagName("bar");
	loadBars(nl2);

	nl2 = root.getElementsByTagName("program");
	loadRecipeMap(nl2);

    }

    public static void loadConstants(NodeList tankNodeList) {

	for (int i = 0; i < tankNodeList.getLength(); i++) {
	    Element e = (Element) tankNodeList.item(i);

	    String file = "constants.txt";
	    FileWriter fw;
	    BufferedWriter bw;
	    PrintWriter outfile;
	    try {
		fw = new FileWriter(file);
		bw = new BufferedWriter(fw);
		outfile = new PrintWriter(bw);
	    } catch (IOException ex) {
		System.err.println(ex.getMessage());
		return;
	    }
	    try {

		outfile
			.println(XMLHelper
				.getIntegerValue(MAX_SAME_POS_TIME, e));
		//outfile.println(XMLHelper.getIntegerValue(CRANE_DOWN_TIME, e));
		//outfile.println(XMLHelper.getIntegerValue(CRANE_UP_TIME, e));
		//outfile.println(XMLHelper.getBooleanValue(DEBUG, e));
		outfile.println(XMLHelper.getIntegerValue(
			MIN_CRANE_DISTANCE_ABS, e));
		outfile.println(XMLHelper.getBooleanValue(CRANE_START_UP, e));
		//outfile.println(XMLHelper.getIntegerValue(COLOR_PARALLEL_TANK,e));

		ABS_conv = XMLHelper.getDoubleValue(ABS, e);
		acceleration = XMLHelper.getDoubleValue(ACCELERATION, e);
		speeds_in_ABS = XMLHelper.getDoubleArray(SPEED_STEPS_IN_ABS, e);
		speeds_in_mm = XMLHelper.getDoubleArray(SPEEDS_IN_MM_S, e);

		// this.MIN_CRANE_DISTANCE_MM ,
		// new Integer((int) (this.this.getDoubleOption(this.ABS) *
		// this.getIntegerOption(this.MIN_CRANE_DISTANCE_ABS))
		crane_down_time = XMLHelper.getIntegerValue(CRANE_DOWN_TIME, e);
		crane_up_time = XMLHelper.getIntegerValue(CRANE_UP_TIME, e);
		parallel_color_tank = XMLHelper.getIntegerValue(
			COLOR_PARALLEL_TANK, e);
	    } catch (ConfigNoValueException ex) {
		System.out.println("no value exception");
	    } catch (ConfigConversionException ex) {
		System.out.println("wrong conversion");
	    }
	    outfile.close();
	}

    }

    public static void loadTankManager(NodeList tankNodeList) {

	// List<Tank> tanks = new ArrayList<Tank>();
	// NodeList nl2 =
	// root.getElementsByTagName("tank");//tanks.getChildNodes();
	/*
	 * String file = "tanks.txt"; FileWriter fw; BufferedWriter bw;
	 * PrintWriter outfile; try { fw = new FileWriter(file); bw = new
	 * BufferedWriter(fw); outfile = new PrintWriter(bw); } catch
	 * (IOException ex) { System.err.println(ex.getMessage()); return; }
	 */
	parallel_tanks = new HashMap<Integer, List<Integer>>();
	ABS_tanks = new HashMap<Integer, Integer>();
	drip_tanks = new HashMap<Integer, Integer>();
	for (int i = 0; i < tankNodeList.getLength(); i++) {
	    Element e = (Element) tankNodeList.item(i);
	    // if("tank".equals(e.getNodeName())) {

	    try {

		// outfile.print(XMLHelper.getStringValue("name", e));

		Integer pos = XMLHelper.getIntegerValue("pos", e);
		Integer pst = XMLHelper.getIntegerValue("pst", e);
		// System.out.println(Arrays.asList(pst));

		if (XMLHelper.getBooleanValue("used", e)) {
		    if (!parallel_tanks.containsKey(pst)) {
			parallel_tanks.put(pst, new ArrayList<Integer>());
		    }
		    parallel_tanks.get(pst).add(pos);
		}
		// System.out.println(pst + ": " + parallel_tanks.get(pst));
		ABS_tanks.put(pos, XMLHelper.getIntegerValue("abs", e));
		// outfile.printf("%d %d\n", pos, pst);
		// XMLHelper.getIntegerValue("drip", e));
		// outfile.print(XMLHelper.getIntegerValue("line", e));
		// outfile.print(XMLHelper.getIntegerValue("tank-nr", e));

		// t.setColorStation(
		// this.getIntegerOption(this.COLOR_PARALLEL_STATION).equals(t.getParallelStation()));

		// tanks.add(t);
		/**
		 * Drip times at the tanks should all be zero.
		 * 
		 */
		drip_tanks.put(pos, XMLHelper.getIntegerValue("drip", e));
		// System.out.println(drip);
	    } catch (ConfigNoValueException ex) {
		System.out.println("no value exception");
	    } catch (ConfigConversionException ex) {
		System.out.println("wrong conversion");
	    }

	}
	// outfile.close();
	// TankManager tm = new TankManager();

	// tm.init(tanks, config);

	// return tm;
    }

    public static void loadColorRecipes(NodeList cpNodeList) {

	// nl2 = root.getElementsByTagName("colorprogram");
	colors = new HashMap<Integer,ColorPrg>();
	
	for (int i = 0; i < cpNodeList.getLength(); i++) {
	    Element e = (Element) cpNodeList.item(i);

	    try {
		Integer p = XMLHelper.getIntegerValue("colorprogram_id", e);
		colors.put(p,new ColorPrg());
		colors.get(p).clear=XMLHelper.getIntegerValue("clearing-time", e); 
		colors.get(p).heat=XMLHelper.getIntegerValue("heating-time", e);	
		//System.out.println(colors);
		// outfile.print(XMLHelper.getStringValue("name", e));
	    } catch (ConfigNoValueException ex) {
		System.out.println("no value exception");
	    } catch (ConfigConversionException ex) {
		System.out.println("wrong conversion");
	    }
	}

    }

    public static void loadCraneManager(NodeList craneNodeList) {

	// DistanceMatrix dm = new DistanceMatrix(config);
	Calculations c;
	c = new Calculations(ABS_conv, acceleration, speeds_in_ABS,
		speeds_in_mm);

	String file = "cranes.txt";
	FileWriter fw;
	BufferedWriter bw;
	PrintWriter outfile;
	try {
	    fw = new FileWriter(file);
	    bw = new BufferedWriter(fw);
	    outfile = new PrintWriter(bw);
	} catch (IOException ex) {
	    System.err.println(ex.getMessage());
	    return;
	}
	for (int l = 0; l < craneNodeList.getLength(); l++) {
	    Element e = (Element) craneNodeList.item(l);
	    try {

		Integer cr_id = XMLHelper.getIntegerValue("crane_id", e);
		Integer min_tk = XMLHelper.getIntegerValue("min_station", e);
		Integer max_tk = XMLHelper.getIntegerValue("max_station", e);

		outfile.printf("%d %d %d %d\n", cr_id, min_tk, max_tk,
			XMLHelper.getIntegerValue("start_position", e));

		double[][] tt = c.calcMatrix(ABS_tanks, min_tk, max_tk);

		for (int i = 0; i < (max_tk - min_tk + 1); i++) {
		    for (int j = 0; j < (max_tk - min_tk + 1); j++) {
			outfile.printf("%2d ", Math.round(tt[i][j]));
		    }
		    outfile.println();
		}

	    } catch (ConfigNoValueException ex) {
		System.out.println("no value exception");
	    } catch (ConfigConversionException ex) {
		System.out.println("wrong conversion");
	    }
	}
	outfile.close();

    }

    public static void loadBars(NodeList barNodeList) {

	/*
	 * int bars_to_use = this.getIntegerOption(this.NUMBER_OF_BARS); int
	 * first_bar = this.getIntegerOption(this.FIRST_BAR);
	 * 
	 * if(bars_to_use == barNodeList.getLength()) {
	 * 
	 * first_bar = 0;
	 * 
	 * } else if(barNodeList.getLength() <= first_bar) { throw new
	 * ConfigException
	 * (this.FIRST_BAR+" value "+first_bar+" >= nr bars ("+barNodeList
	 * .getLength()+")!!!"); } else if((first_bar + bars_to_use) >
	 * barNodeList.getLength()) { if(first_bar == 0) { bars_to_use =
	 * barNodeList.getLength(); } else { throw new
	 * ConfigException(this.FIRST_BAR
	 * +"+"+this.NUMBER_OF_BARS+" = "+(first_bar
	 * +bars_to_use)+" >= nr bars ("+barNodeList.getLength()+")!!!"); } }
	 */

	String file = "bars.txt";
	FileWriter fw;
	BufferedWriter bw;
	PrintWriter outfile;
	try {
	    fw = new FileWriter(file);
	    bw = new BufferedWriter(fw);
	    outfile = new PrintWriter(bw);
	} catch (IOException ex) {
	    System.err.println(ex.getMessage());
	    return;
	}
Integer null_time=Integer.MAX_VALUE;
	for (int i = 0; i < barNodeList.getLength(); i++) {

	    Element e = (Element) barNodeList.item(i);

	    try {
		Integer t = XMLHelper.getIntegerValue("loading_time", e);
		if (t < null_time)
		    null_time=t;
	    } catch (ConfigNoValueException ex) {
		System.out.println("no value exception");
	    } catch (ConfigConversionException ex) {
		System.out.println("wrong conversion");
	    }
	}
	color_bar = new HashMap<Integer, Integer>();
	for (int i = 0; i < barNodeList.getLength(); i++) {

	    Element e = (Element) barNodeList.item(i);

	    try {
		Integer id = XMLHelper.getIntegerValue("bar_id", e);
		color_bar.put(id, XMLHelper.getIntegerValue("color_program_id",
			e));
		outfile.printf("%d %d %d\n", id,
		// XMLHelper.getIntegerValue("bar_number", e),
			// XMLHelper.getIntegerValue("program_id", e),
			XMLHelper.getIntegerValue("loading_time", e)-null_time, XMLHelper
				.getIntegerValue("orig_out_line_time", e)-null_time);
		// outfile.print(XMLHelper.getStringValue("charge_code", e));
	    } catch (ConfigNoValueException ex) {
		System.out.println("no value exception");
	    } catch (ConfigConversionException ex) {
		System.out.println("wrong conversion");
	    }
	    // (XMLHelper.hasUniqueElement("orig_out_line_time", e)) {
	}
	//System.out.println(color_bar);
	outfile.close();

    }

    public static void loadRecipeMap(NodeList prgNodeList) {
	String file = "recipes.txt";
	FileWriter fw;
	BufferedWriter bw;
	PrintWriter outfile;
	try {
	    fw = new FileWriter(file);
	    bw = new BufferedWriter(fw);
	    outfile = new PrintWriter(bw);
	} catch (IOException ex) {
	    System.err.println(ex.getMessage());
	    return;
	}

	for (int i = 0; i < prgNodeList.getLength(); i++) {
	    Element e = (Element) prgNodeList.item(i);
	    Integer p = 0;
	    try {
		p = XMLHelper.getIntegerValue("program_id", e);
	    } catch (ConfigNoValueException ex) {
		System.out.println("no value exception");
	    } catch (ConfigConversionException ex) {
		System.out.println("wrong conversion");
	    }
	    // System.out.print("P: "+p.getId());
	    // System.out.print(" STGS: ");
	    NodeList stages = e.getElementsByTagName("stage");

	    // List<Stage> stg_list = new ArrayList<Stage>();
	    // System.out.println(stages.getLength());
	    int stg_nr=0;
	    for (int j = 0; j < stages.getLength(); j++) {
		Element stg = (Element) stages.item(j);
		try {
		    stg_nr = Integer.parseInt(stg.getAttribute("stg_nr"));
		    int mintime = Integer.parseInt(stg.getAttribute("min"));
		    int maxtime = Integer.parseInt(stg.getAttribute("max"));
		    int parallel_tank_nr = Integer.parseInt(stg
			    .getAttribute("parallel"));

		    // boolean spray =
		    // Boolean.parseBoolean(stg.getAttribute("spray"));
		    boolean tray = Boolean.parseBoolean(stg
			    .getAttribute("tray"));

		    int drippingtime = Integer.parseInt(stg
			    .getAttribute("drip"));
		    // Tank tank = tm.getByParallelStation(parallel_tank_nr);

		    // if(tank == null) throw new
		    // ConfigException("No tank for  parallel tank: "+parallel_tank_nr);

		    List<Integer> pl = parallel_tanks.get(parallel_tank_nr);
		    int which = r.nextInt(pl.size());
		    Integer tank = pl.get(which);
//		    System.out.print(tank+" "+parallel_tank_nr +" "+ parallel_color_tank+" ");
		    int liftTime = 0;
		    liftTime += crane_up_time;
		    liftTime += drippingtime > 0 ? drippingtime : drip_tanks
			    .get(tank);// tank.getDrippingTime();
		    liftTime += tray ? 3 : 0;

		    int sinkTime = crane_down_time;

		    boolean colorTank = (parallel_tank_nr == parallel_color_tank);

		    if (colorTank) {
//			System.out.print(color_bar+" "+colors+" ");
			sinkTime += colors.get(color_bar.get(p)).heat;
			liftTime += colors.get(color_bar.get(p)).clear;
			
		    }
//System.out.println(sinkTime+" "+liftTime);
		    outfile.printf("%d %d %d %d" + " %d %d %d\n", p, stg_nr,
			    tank, mintime, maxtime, sinkTime, liftTime);
		} catch (Exception ex) {
		    System.out.println("parse Int in stage " + ex.getMessage()
			    + stg);
		}
	    }
	    outfile.printf("%d %d 57 0 0 14 20\n", p, stg_nr+1);

	    // outfile.println("");
	}
	outfile.close();

    }

    public static void main(String[] args) {
	r = new Random(1);
	loadFromXML(args[0]);

    }

}
