/**
 * class and methods for reading from XML files
 */


import java.util.ArrayList;
import java.util.Formatter;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;


import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;





class ConfigConversionException extends Exception {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public ConfigConversionException(String string) {
		super(string);
	}

}



class ConfigException extends Exception {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public ConfigException(String string) {
		super(string);
	}

}



class ConfigNoValueException extends Exception {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public ConfigNoValueException(String string) {
		super(string);
	}

}

public class XMLHelper {

	/*
	public static void writeXMLDocument(Document doc, String filename) 
	{
		try {

			OutputStream out = new FileOutputStream(new File(filename));

			Properties props = OutputPropertiesFactory.getDefaultMethodProperties(Method.XML);

			props.setProperty(OutputKeys.INDENT, "yes");

			props.setProperty(OutputPropertiesFactory.S_KEY_INDENT_AMOUNT, "4");
			props.setProperty(OutputPropertiesFactory.S_KEY_LINE_SEPARATOR, "\n");

			Serializer ser = SerializerFactory.getSerializer(props);

			ser.setOutputStream(out);
			DOM3Serializer d3ser = (DOM3Serializer) ser.asDOM3Serializer();
			d3ser.setNewLine(new char[]{'\n'});
			d3ser.serializeDOM3(doc);
			ser.reset();

		} catch (Exception e) {
			e.printStackTrace();
		} 

	}
	 */
	//public static void 



	public static Element newElement(Document doc, Element parent, String name)
	{
		return newElement(doc, parent, name, null);
	}

	public static Element newStringElement(Document doc, Element parent, String name, String value)
	{
		return newElement(doc, parent, name, value);
	}
	public static Element newIntegerElement(Document doc, Element parent, String name, Integer value)
	{
		return newElement(doc, parent, name, value.toString());
	}

	private static Element newElement(Document doc, Element parent, String name, String value)
	{
		if(doc != null && name != null && !"".equals(name)){
			Element e = doc.createElement(name);
			if(value != null && !"".equals(value))
			{
				e.appendChild(doc.createTextNode(value));
			}
			if(parent != null) {
				parent.appendChild(e);
			}
			return e;
		} else 
		{
			return null;
		}
	}

	public static Node getNode(String name, Element elm) throws ConfigNoValueException
	{
		NodeList nl = elm.getElementsByTagName(name);
		if(nl.getLength() < 1)
			return null;

		Node child = nl.item(0);
		return child;
	}
	public static String getStringValue(String name, Element elm) throws ConfigNoValueException
	{
		NodeList nl = elm.getElementsByTagName(name);
		if(nl.getLength() != 1)
			throw new ConfigNoValueException("the element "+name+" in "+elm.getNodeName()+" could not be found");

		Node child = nl.item(0);
		return child.getTextContent().trim();
	}
	public static String getNotNullStringValue(String name, Element elm) throws ConfigNoValueException
	{
		String text = XMLHelper.getStringValue(name, elm);
		if("".equals(text))
			throw new ConfigNoValueException("The element "+name+" does not contain a value!");

		return text;
	}

	public static Integer getIntegerValue(String name, Element elm) throws ConfigConversionException, ConfigNoValueException
	{
		String text = XMLHelper.getNotNullStringValue(name, elm);
		if(!text.matches("[-+]?\\d\\d*"))
		{
			System.out.println("NONOTNO");
			throw new ConfigConversionException("The element "+name+" in "+elm.getNodeName()+" does not contain an integer but: "+text+"");
		}

		return Integer.parseInt(text);
	}

	public static Long getLongValue(String name, Element elm) throws ConfigConversionException, ConfigNoValueException
	{
		String text = XMLHelper.getNotNullStringValue(name, elm);
		if(!text.matches("[-+]?\\d\\d*"))
		{
			System.out.println("NONOTNO");
			throw new ConfigConversionException("The element "+name+" in "+elm.getNodeName()+" does not contain an integer but: "+text+"");
		}

		return Long.parseLong(text);
	}


	public static Double getDoubleValue(String name, Element elm) throws ConfigConversionException, ConfigNoValueException
	{
		String text = XMLHelper.getNotNullStringValue(name, elm);
		if(!text.matches("[+-]?\\d\\d*\\.?\\d*"))
		{
			throw new ConfigConversionException("The element "+name+" in "+elm.getNodeName()+" does not contain an double but: "+text+"");
		}

		return Double.parseDouble(text);
	}

	public static Boolean getBooleanValue(String name, Element elm) throws ConfigConversionException, ConfigNoValueException
	{
		String text = XMLHelper.getNotNullStringValue(name, elm).trim();

		if(!text.matches("true|false"))
		{
			throw new ConfigConversionException("The element "+name+" in "+elm.getNodeName()+" does not contain a boolean(true or false) but: "+text+"");
		}

		return Boolean.parseBoolean(text);

	}

	public static List<String> getStringArray(String name, Element elm) throws ConfigNoValueException
	{
		List<String> strings = new ArrayList<String>();

		String liststr = XMLHelper.getStringValue(name, elm);
		if(!liststr.equals("")){
			String[] values = liststr.split(",");
			for(String str : values)
			{
				strings.add(str.trim());
			}
		}

		return strings;
	}

	public static List<Integer> getIntegerArray(String name, Element elm) throws ConfigConversionException, NumberFormatException, ConfigNoValueException
	{
		List<Integer> values = new ArrayList<Integer>();
		for(String str : XMLHelper.getStringArray(name, elm))
		{
			if(!str.matches("[+-]?\\d\\d*"))
			{
				throw new ConfigConversionException("The element "+name+" in "+elm.getNodeName()+" does not contain an integer array but:"+XMLHelper.getStringValue(name, elm)+"");
			}
			values.add(Integer.parseInt(str));
		}
		return values;
	}

	public static List<Double> getDoubleArray(String name, Element elm) throws ConfigConversionException, NumberFormatException, ConfigNoValueException
	{
		List<Double> values = new ArrayList<Double>();
		for(String str : XMLHelper.getStringArray(name, elm))
		{
			if(!str.matches("[+-]?\\d\\d*\\.?\\d*"))
			{
				throw new ConfigConversionException("The element "+name+" in "+elm.getNodeName()+" does not contain an double array but:"+XMLHelper.getStringValue(name, elm)+"");
			}
			values.add(Double.parseDouble(str));
		}
		return values;
	}


	public static Integer getIntegerAttribute(String name, Element elm) throws ConfigConversionException, ConfigNoValueException
	{
		String value = elm.getAttribute(name);
		if(value == null){

			throw new ConfigNoValueException("The attribute "+name+" in "+elm.getNodeName()+" does not exist!");

		} else  if(!value.matches("[+-]?\\d\\d*")){

			throw new ConfigConversionException("The attribute "+name+" in "+elm.getNodeName()+" does not contain an integer:"+value+"");

		}

		return Integer.parseInt(value);
	}




	public static void printElement(Node current, int indent)
	{

		String ind = "";
		for (int j = 0; j < indent; j++) {
			ind += " --> ";
		}
		Formatter fm = new Formatter(System.out);
		fm.format("%1$s%2$-6s:%3$10s : %4$s\n", 
				ind,  
				XMLHelper.getNodeTypeName(current), 
				current.getNodeName(), 
				current.getNodeValue(),
				current.getChildNodes().getLength());

		indent++;

		NodeList nl = current.getChildNodes();
		for (int i = 0; i < nl.getLength(); i++) {
			XMLHelper.printElement(nl.item(i), indent);
		}

	}


	public static String getNodeTypeName(Node n)
	{
		String res = "NAN1";
		switch(n.getNodeType())
		{
		case Node.ATTRIBUTE_NODE: res = "ATT"; break;
		case Node.CDATA_SECTION_NODE: res = "CDATA"; break;
		case Node.COMMENT_NODE: res = "COMMENT"; break;
		case Node.DOCUMENT_FRAGMENT_NODE: res = "DOC_FRAC"; break;
		case Node.DOCUMENT_TYPE_NODE: res = "DOC_TYPE"; break;
		case Node.ELEMENT_NODE: res = "ELM"; break;
		case Node.ENTITY_NODE: res = "ENT"; break;
		case Node.ENTITY_REFERENCE_NODE: res = "ENT_REF"; break;
		case Node.NOTATION_NODE: res = "NOTATION"; break;
		case Node.PROCESSING_INSTRUCTION_NODE: res = "PIN"; break;
		case Node.TEXT_NODE: res = "TEXT"; break;
		case Node.DOCUMENT_NODE: res = "DOC_NODE"; break;
		//case Node.: res = "";
		//default: res = "NAN";
		}

		return res;
	}

	public static Document nodeToDocument(Node node)
	{

		Document doc = null;
		try {
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			//DOMImplementation di = db.getDOMImplementation();

			doc = db.newDocument();

			doc.adoptNode(node);
			doc.appendChild(node);

			//XMLHelper.printElement(doc, 1);


		} catch (DOMException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return doc;

	}

	public static Element copyElements(Document doc, Node current)
	{

		Element elm = doc.createElement(current.getNodeName());

		NodeList nl = current.getChildNodes();

		boolean all_text_nodes = true;
		for (int i = 0; i < nl.getLength(); i++) {
			if(nl.item(i).getNodeType() != Node.TEXT_NODE){
				all_text_nodes = false;
			}	
		}

		if(all_text_nodes)
		{
			StringBuilder sb = new StringBuilder();

			for (int i = 0; i < nl.getLength(); i++) {

				//if(!nl.item(i).getNodeValue().matches("^[ \\t]*$")){
				//System.out.println("APPEND: --"+nl.item(i).getNodeValue()+"--");
				sb.append(nl.item(i).getTextContent().trim());
				//}

			}
			elm.setTextContent(sb.toString());
			//elm.setTextContent(nl.item(0).getNodeValue().trim());
		} else {

			for (int i = 0; i < nl.getLength(); i++) {
				if(nl.item(i).getNodeType() == Node.ELEMENT_NODE){
					elm.appendChild(XMLHelper.copyElements(doc, nl.item(i)));
				}		
			}
		}

		return elm;

	}

	public static boolean hasUniqueElement(String name, Element elm) {
		NodeList nl = elm.getElementsByTagName(name);
		return nl.getLength() == 1;	
	}

	public static boolean hasElement(String name, Element elm) {
		NodeList nl = elm.getElementsByTagName(name);
		return nl.getLength() >= 1;
		
	}




}
